/* Copyright (c) Jasper van de Gronde, 2004
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above
 *     copyright notice, this list of conditions and the
 *     following disclaimer.
 *  2. Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Drawing;
using System.Windows.Forms;

/** <summary>Frame to issue simple queries</summary>
 * <remarks>Shows a frame with space to enter a query and a listview that shows the response.</remarks>
 */

class QueryFrame : Form {
  private TextBox queryText = new TextBox();
  private Button queryButton = new Button();
  private Label statusLabel = new Label();
  private ListView resultList = new ListView();

  private FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
  private OpenFileDialog openFileDialog = new OpenFileDialog();

  private RdfDbConnection rdfdb;
  private MDDB mddb;

  public QueryFrame() {
    rdfdb = new RdfDbConnection();
    mddb = new MDDB(rdfdb);

    Layout += new LayoutEventHandler(QueryFrame_Layout);
    queryButton.Click += new EventHandler(queryButton_Clicked);

    buildMenu();

    Text = "RDFDB C# Frontend";
    queryButton.Text = "Query";
    resultList.View = View.Details;
    statusLabel.Text = "Status: I haven't done anything yet";
    statusLabel.Size = new Size (statusLabel.PreferredWidth, statusLabel.PreferredHeight);

    folderBrowserDialog.Description = "Pick a directory to add to MDDB.";
    folderBrowserDialog.ShowNewFolderButton = false;
    folderBrowserDialog.RootFolder = Environment.SpecialFolder.Personal;
    openFileDialog.Title = "Select some files to add to MDDB.";
    openFileDialog.Multiselect = true;
    openFileDialog.RestoreDirectory = true;

    Controls.Add(queryText);
    Controls.Add(queryButton);
    Controls.Add(statusLabel);
    Controls.Add(resultList);
  }

  private void processResultRecord(string[] record) {
    if ( record.Length <= 0 ) return;

    if ( resultList.Columns.Count > 0 ) {
      ListViewItem item = new ListViewItem(record);
      resultList.Items.Add(item);
    }
    else {
      int endindex = (record[record.Length-1].Length > 0 ? record.Length : record.Length-1);
      for(int i = 0; i < endindex; ++i) {
        if ( record[i].Length > 1 && record[i][0] == '?' ) {
          record[i] = record[i].Remove(0,1);
        }
        resultList.Columns.Add(record[i], 100, HorizontalAlignment.Left);
      }
    }
  }

  private void queryButton_Clicked(object sender, EventArgs e) {
    statusLabel.Text = "Status: running...";
    statusLabel.Size = new Size (statusLabel.PreferredWidth, statusLabel.PreferredHeight);
    resultList.Columns.Clear();
    resultList.Items.Clear();
    int errcode;
    if ( (errcode=rdfdb.runQuery(queryText.Text, new RdfDbConnection.processRecord(processResultRecord))) == 0 ) {
      statusLabel.Text = "Status: finished (retrieved " + resultList.Items.Count + " records)";
      statusLabel.Size = new Size (statusLabel.PreferredWidth, statusLabel.PreferredHeight);
    }
    else {
      statusLabel.Text = "Status: failed! (error code " + errcode + ")";
      statusLabel.Size = new Size (statusLabel.PreferredWidth, statusLabel.PreferredHeight);
    }
  }

  private void QueryFrame_Layout(object sender, LayoutEventArgs e) {
    Size size = this.ClientSize;
    int cumHeight = 0;

    queryText.SetBounds(0, cumHeight, size.Width, queryText.Size.Height);
    cumHeight += queryText.Size.Height;

    queryButton.SetBounds(0, cumHeight, queryButton.Size.Width, queryButton.Size.Height);
    statusLabel.SetBounds(queryButton.Size.Width, cumHeight + (Math.Max(queryButton.Size.Height, statusLabel.PreferredHeight)-statusLabel.PreferredHeight)/2, statusLabel.PreferredWidth, statusLabel.PreferredHeight);
    cumHeight += Math.Max(queryButton.Size.Height, statusLabel.Size.Height);

    resultList.SetBounds(0, cumHeight, size.Width, size.Height-cumHeight);
    cumHeight += resultList.Size.Height;
  }

  private void addDirectory(object sender, EventArgs e) {
    DialogResult result = folderBrowserDialog.ShowDialog();
    if( result == DialogResult.OK )
    {
      mddb.addDirectory(folderBrowserDialog.SelectedPath);
    }
  }

  private void addFiles(object sender, EventArgs e) {
    DialogResult result = openFileDialog.ShowDialog();
    if( result == DialogResult.OK )
    {
      mddb.addFiles(openFileDialog.FileNames);
    }
  }

  private void buildMenu() {
    MainMenu mainmenu = new MainMenu(new MenuItem[]
    {
      new MenuItem("Add directory", new EventHandler(addDirectory)),
      new MenuItem("Add files", new EventHandler(addFiles))
    });
    Menu = mainmenu;
  }
}
