/* Copyright (c) Jasper van de Gronde, 2004
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above
 *     copyright notice, this list of conditions and the
 *     following disclaimer.
 *  2. Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System.IO;
using System.Net;
using System.Net.Sockets;

/** <summary>Handles a connection with RDFDB</summary>
 * <remarks>Sets up a connection with a running instance of RDFDB and lets
 * the user of this class issue queries. Results are returned through delegates.</remarks>
 */

class RdfDbConnection {
  //private TcpClient tcpclient;
  private Socket tcpsocket = null;
  private NetworkStream stream = null;
  private StreamWriter outstr = null;
  private StreamReader instr = null;
  private readonly IPEndPoint ipe;

  public delegate void processRecord(string[] record);
  
  public RdfDbConnection() {
    ipe = new IPEndPoint(IPAddress.Loopback, 7001);
  }

  public RdfDbConnection(string hostname) {
    IPHostEntry iphe = Dns.Resolve(hostname);
    if ( iphe.AddressList.Length > 0 ) {
      ipe = new IPEndPoint(iphe.AddressList[0], 7001);
    }
  }

  public RdfDbConnection(string hostname, int port) {
    IPHostEntry iphe = Dns.Resolve(hostname);
    if ( iphe.AddressList.Length > 0 ) {
      ipe = new IPEndPoint(iphe.AddressList[0], port);
    }
  }

  ~RdfDbConnection() {
    close();
  }

  public bool connect() {
    if ( connected() ) close();
    try {
      tcpsocket = new Socket(ipe.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
      tcpsocket.Connect(ipe);
      if ( tcpsocket.Connected ) {
        stream = new NetworkStream(tcpsocket);
        outstr = new StreamWriter(stream);
        instr = new StreamReader(stream);
      }
      return true;
    }
    catch(System.Net.Sockets.SocketException) {
      tcpsocket = null;
      return false;
    }
    catch(System.IO.IOException) {
      close();
      return false;
    }
  }

  public bool connected() {
    return tcpsocket != null && tcpsocket.Connected;
  }

  public void close() {
    if ( connected() ) {
      tcpsocket.Close();
    }
    tcpsocket = null;
  }

  /// <returns>Returns 0 on success, values below 0 for server errors, values above 0 for client errors.</returns>
  public int addTriple(string db, string subject, string predicate, string obj) {
    return runQuery("insert into " + db + " (" + predicate + " " + subject + " " + obj + ")");
  }

  /// <returns>Returns 0 on success, values below 0 for server errors, values above 0 for client errors.</returns>
  public int runQuery(string query) {
    if ( !connect() ) return 1;

    outstr.Write(query + " </>\xA");
    outstr.Flush();
    string line;
    while(true) {
      line = instr.ReadLine();
      if ( line.StartsWith("#") ) {
        close();
        int i = line.IndexOf(':');
        if ( i >= 1 ) {
          return System.Convert.ToInt32(line.Substring(1, i-1));
        }
        else {
          return System.Convert.ToInt32(line.Substring(1));
        }
      }
      // Any output is discarded.
    }
  }

  /// <returns>Returns 0 on success, values below 0 for server errors, values above 0 for client errors.</returns>
  public int runQuery(string query, processRecord process) {
    if ( !connect() ) return 1;

    outstr.Write(query + " </>\xA");
    outstr.Flush();
    string line;
    while(true) {
      line = instr.ReadLine();
      if ( line.StartsWith("#") ) {
        close();
        int i = line.IndexOf(':');
        if ( i >= 1 ) {
          return System.Convert.ToInt32(line.Substring(1, i-1));
        }
        else {
          return System.Convert.ToInt32(line.Substring(1));
        }
      }
      process(line.Split('\t'));
    }
  }
}
